@@ -1,23 +1,74 @@
+//! Module for parsing and analyzing autopkgtest logs.
+//!
+//! This module provides functionality for parsing and analyzing logs from
+//! autopkgtest, the Debian automated testing framework. It can identify and
+//! extract test results, errors, and other information from autopkgtest logs.
+
 use crate::lines::Lines;
 use crate::problems::autopkgtest::*;
 use crate::{Match, Problem, SingleLineMatch};
 use std::collections::HashMap;
 
+/// Type alias for autopkgtest failure result
+pub type AutopkgtestResult = (Option<Box<dyn Match>>, Option<Box<dyn Problem>>);
+
+/// Type alias for autopkgtest detailed result with test info
+pub type AutopkgtestDetailedResult = (
+    Option<Box<dyn Match>>,
+    Option<String>,
+    Option<Box<dyn Problem>>,
+    Option<String>,
+);
+
+/// Represents different types of log packets in autopkgtest output.
+///
+/// Each variant corresponds to a specific type of log entry in autopkgtest output,
+/// such as test results, source information, or error messages.
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub enum Packet<'a> {
+    /// Source information marker in the log.
     Source,
+
+    /// Summary section marker in the log.
     Summary,
+
+    /// Beginning of test output for a specific test.
     TestBeginOutput(&'a str),
+
+    /// End of test output for a specific test.
     TestEndOutput(&'a str),
+
+    /// Test results section for a specific test.
     Results(&'a str),
+
+    /// Standard error output from a specific test.
     Stderr(&'a str),
+
+    /// Testbed setup information for a specific test.
     TestbedSetup(&'a str),
+
+    /// General test output with test name and status.
     TestOutput(&'a str, &'a str),
+
+    /// Error message in the log.
     Error(&'a str),
+
+    /// Other unrecognized log entries.
     Other(&'a str),
 }
 
-fn parse_autopgktest_line(line: &str) -> Option<(&str, Packet)> {
+/// Parses a single line from an autopkgtest log.
+///
+/// This function extracts the timestamp and message content from a log line
+/// and categorizes it into a specific type of packet.
+///
+/// # Arguments
+/// * `line` - The log line to parse
+///
+/// # Returns
+/// An option containing a tuple of (timestamp, packet) if the line is a valid autopkgtest log line,
+/// or None if the line doesn't match the expected format
+fn parse_autopgktest_line(line: &str) -> Option<(&str, Packet<'_>)> {
     let (timestamp, message) =
         match lazy_regex::regex_captures!(r"autopkgtest \[([0-9:]+)\]: (.*)", line) {
             Some((_, timestamp, message)) => (timestamp, message),
@@ -53,11 +104,21 @@ fn parse_autopgktest_line(line: &str) -> Option<(&str, Packet)> {
     }
 }
 
+/// Represents the result of an individual autopkgtest test.
+///
+/// This enum represents the possible outcomes of an autopkgtest test case.
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum TestResult {
+    /// Test passed successfully.
     Pass,
+
+    /// Test failed.
     Fail,
+
+    /// Test was skipped.
     Skip,
+
+    /// Test produced inconsistent results (sometimes passes, sometimes fails).
     Flaky,
 }
 
@@ -75,24 +136,56 @@ impl std::str::FromStr for TestResult {
     }
 }
 
+/// Summary of an autopkgtest test result.
+///
+/// This structure represents a summary of an individual test result from an autopkgtest run,
+/// including the test name, result status, and additional information.
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct Summary {
+    /// The line offset of this summary in the log.
     pub offset: usize,
+
+    /// The name of the test.
     pub name: String,
+
+    /// The result of the test (pass, fail, skip, or flaky).
     pub result: TestResult,
+
+    /// Optional reason for the result, especially useful for failures.
     pub reason: Option<String>,
+
+    /// Additional information about the test result.
     pub extra: Vec<String>,
 }
 
 impl Summary {
+    /// Gets the line offset of this summary in the log.
+    ///
+    /// # Returns
+    /// The zero-based line offset
     pub fn offset(&self) -> usize {
         self.offset
     }
+
+    /// Gets the line number of this summary in the log.
+    ///
+    /// # Returns
+    /// The one-based line number
     pub fn lineno(&self) -> usize {
         self.offset + 1
     }
 }
 
+/// Parses autopkgtest summary lines to extract test results.
+///
+/// This function analyzes the summary section of an autopkgtest log
+/// and extracts information about individual test results.
+///
+/// # Arguments
+/// * `lines` - Vector of log lines from the summary section
+///
+/// # Returns
+/// A vector of `Summary` structures, one for each identified test result
 pub fn parse_autopkgtest_summary(lines: Vec<&str>) -> Vec<Summary> {
     let mut i = 0;
     let mut ret = vec![];
@@ -138,13 +231,28 @@ pub fn parse_autopkgtest_summary(lines: Vec<&str>) -> Vec<Summary> {
     ret
 }
 
+/// Represents different field types in an autopkgtest log.
+///
+/// This enum is used internally to categorize different sections of the log
+/// based on their content type and associated test.
 #[derive(Debug, PartialEq, Eq, Clone, std::hash::Hash)]
 enum Field {
+    /// General output for a specific test.
     Output(String),
-    FieldOutput(String, String),
+
+    /// Specific field output for a test.
+    SpecificOutput(String, String),
+
+    /// Standard error output for a test.
     Stderr(String),
+
+    /// Results output for a test.
     Results(String),
+
+    /// Testbed preparation output for a test.
     PrepareTestbed(String),
+
+    /// Summary section.
     Summary,
 }
 
@@ -155,7 +263,7 @@ impl std::fmt::Display for Field {
             Field::Stderr(testname) => write!(f, "stderr for test {}", testname),
             Field::Results(testname) => write!(f, "results for test {}", testname),
             Field::PrepareTestbed(testname) => write!(f, "testbed setup for test {}", testname),
-            Field::FieldOutput(testname, field) => {
+            Field::SpecificOutput(testname, field) => {
                 write!(f, "{} for test {}", field, testname)
             }
             Field::Summary => write!(f, "summary"),
@@ -170,21 +278,14 @@ impl Field {
             Field::Stderr(testname) => Some(testname),
             Field::Results(testname) => Some(testname),
             Field::PrepareTestbed(testname) => Some(testname),
-            Field::FieldOutput(testname, _) => Some(testname),
+            Field::SpecificOutput(testname, _) => Some(testname),
             Field::Summary => None,
         }
     }
 }
 
 /// Find the autopkgtest failure in output.
-pub fn find_autopkgtest_failure_description(
-    mut lines: Vec<&str>,
-) -> (
-    Option<Box<dyn Match>>,
-    Option<String>,
-    Option<Box<dyn Problem>>,
-    Option<String>,
-) {
+pub fn find_autopkgtest_failure_description(mut lines: Vec<&str>) -> AutopkgtestDetailedResult {
     let mut test_output: HashMap<Field, (Vec<String>, usize)> = HashMap::new();
     let mut current_field: Option<Field> = None;
     let mut it = lines.iter().enumerate().peekable();
@@ -252,13 +353,12 @@ pub fn find_autopkgtest_failure_description(
                                 .get(&field)
                                 .map_or(vec![], |x| x.0.iter().map(|x| x.as_str()).collect()),
                         );
-                        if let Some(error) = error {
-                            assert!(r#match.is_some());
-                            let description = r#match.as_ref().unwrap().line();
+                        if let (Some(m), Some(error)) = (r#match, error) {
+                            let description = m.line().to_string();
                             return (
                                 Some(Box::new(SingleLineMatch::from_lines(
                                     &lines,
-                                    test_output.get(&field).unwrap().1 + r#match.unwrap().offset(),
+                                    test_output.get(&field).unwrap().1 + m.offset(),
                                     Some("direct regex"),
                                 )) as Box<dyn Match>),
                                 last_test.map(|x| x.to_owned()),
@@ -274,23 +374,23 @@ pub fn find_autopkgtest_failure_description(
                         let (r#match, error) =
                             crate::common::find_build_failure_description(lines.clone());
                         if let Some(error) = error {
-                            let description = r#match.as_ref().unwrap().line();
+                            let description = r#match.as_ref().unwrap().line().to_string();
                             return (r#match, last_test, Some(error), Some(description));
                         }
                     }
 
                     if testbed_failure_reason == "cannot send to testbed: [Errno 32] Broken pipe" {
                         let (r#match, error) = find_testbed_setup_failure(lines.clone());
-                        if error.is_some() && r#match.is_some() {
-                            let description = r#match.as_ref().unwrap().line();
-                            return (r#match, last_test, error, Some(description));
+                        if let (Some(m), Some(e)) = (r#match, error) {
+                            let description = m.line().to_string();
+                            return (Some(m), last_test, Some(e), Some(description));
                         }
                     }
                     if testbed_failure_reason == "apt repeatedly failed to download packages" {
                         let (r#match, error) = crate::apt::find_apt_get_failure(lines.clone());
-                        if error.is_some() && r#match.is_some() {
-                            let description = r#match.as_ref().unwrap().line();
-                            return (Some(r#match.unwrap()), last_test, error, Some(description));
+                        if let (Some(m), Some(e)) = (r#match, error) {
+                            let description = m.line().to_string();
+                            return (Some(m), last_test, Some(e), Some(description));
                         }
                         return (
                             Some(Box::new(SingleLineMatch::from_lines(
@@ -324,12 +424,12 @@ pub fn find_autopkgtest_failure_description(
                 {
                     let (r#match, error) =
                         crate::common::find_build_failure_description(lines[..i].to_vec());
-                    let description = r#match.as_ref().unwrap().line();
-                    if error.is_some() && r#match.is_some() {
+                    if let (Some(m), Some(e)) = (r#match, error) {
+                        let description = m.line().to_string();
                         return (
-                            r#match,
+                            Some(m),
                             last_test.map(|x| x.to_owned()),
-                            error,
+                            Some(e),
                             Some(description),
                         );
                     }
@@ -347,12 +447,12 @@ pub fn find_autopkgtest_failure_description(
                 if msg == "unexpected error:" {
                     let (r#match, error) =
                         crate::common::find_build_failure_description(lines[(i + 1)..].to_vec());
-                    let description = r#match.as_ref().unwrap().line();
-                    if error.is_some() && r#match.is_some() {
+                    if let (Some(m), Some(e)) = (r#match, error) {
+                        let description = m.line().to_string();
                         return (
-                            r#match,
+                            Some(m),
                             last_test.map(|x| x.to_owned()),
-                            error,
+                            Some(e),
                             Some(description),
                         );
                     }
@@ -367,22 +467,20 @@ pub fn find_autopkgtest_failure_description(
                             .map(|x| x.as_str())
                             .collect(),
                     );
-                    if error.is_some()
-                        && r#match.is_some()
-                        && test_output.contains_key(current_field)
-                    {
-                        let description = r#match.as_ref().unwrap().line();
-                        return (
-                            Some(Box::new(SingleLineMatch::from_lines(
-                                &lines,
-                                test_output.get(current_field).unwrap().1
-                                    + r#match.unwrap().offset(),
-                                Some("direct regex"),
-                            )) as Box<dyn Match>),
-                            last_test.map(|x| x.to_owned()),
-                            error,
-                            Some(description),
-                        );
+                    if let (Some(m), Some(e)) = (r#match, error) {
+                        if let Some(test_output_entry) = test_output.get(current_field) {
+                            let description = m.line().to_string();
+                            return (
+                                Some(Box::new(SingleLineMatch::from_lines(
+                                    &lines,
+                                    test_output_entry.1 + m.offset(),
+                                    Some("direct regex"),
+                                )) as Box<dyn Match>),
+                                last_test.map(|x| x.to_owned()),
+                                Some(e),
+                                Some(description),
+                            );
+                        }
                     }
                 }
                 if msg == "autopkgtest" && lines[i + 1].trim_end() == ": error cleaning up:" {
@@ -461,7 +559,7 @@ pub fn find_autopkgtest_failure_description(
                     }
                     Packet::TestOutput(testname, field) => {
                         current_field =
-                            Some(Field::FieldOutput(testname.to_owned(), field.to_owned()));
+                            Some(Field::SpecificOutput(testname.to_owned(), field.to_owned()));
                     }
                     _ => {}
                 }
@@ -543,9 +641,9 @@ pub fn find_autopkgtest_failure_description(
             if !stderr_lines.is_empty() {
                 (r#match, error) =
                     crate::common::find_build_failure_description(stderr_lines.clone());
-                if r#match.is_some() && stderr_offset.is_some() {
-                    offset = Some(r#match.as_ref().unwrap().offset() + stderr_offset.unwrap());
-                    description = Some(r#match.as_ref().unwrap().line());
+                if let (Some(m), Some(so)) = (r#match.as_ref(), stderr_offset) {
+                    offset = Some(m.offset() + so);
+                    description = Some(m.line().to_string());
                 } else if stderr_lines.len() == 1
                     && lazy_regex::regex_is_match!(
                         r"QStandardPaths: XDG_RUNTIME_DIR not set, defaulting to \'(.*)\'",
@@ -553,7 +651,7 @@ pub fn find_autopkgtest_failure_description(
                     )
                 {
                     error = Some(Box::new(XDGRunTimeNotSet) as Box<dyn Problem>);
-                    description = Some(stderr_lines[0].to_owned());
+                    description = Some(stderr_lines[0].to_string());
                     offset = stderr_offset;
                 } else {
                     if let Some(stderr_offset) = stderr_offset {
@@ -567,7 +665,7 @@ pub fn find_autopkgtest_failure_description(
                 (offset, description) = if let Some(r#match) = r#match.as_ref() {
                     (
                         Some(summary_offset + packet.offset() + r#match.offset()),
-                        Some(r#match.line()),
+                        Some(r#match.line().to_string()),
                     )
                 } else {
                     (None, None)
@@ -598,17 +696,17 @@ pub fn find_autopkgtest_failure_description(
                 || (vec![], None),
                 |x| (x.0.iter().map(|x| x.as_str()).collect(), Some(x.1)),
             );
-            if !output_lines.is_empty() && output_offset.is_some() {
+            if let (false, Some(offset)) = (output_lines.is_empty(), output_offset) {
                 let (r#match, error) = crate::apt::find_apt_get_failure(output_lines);
-                if error.is_some() && r#match.is_some() {
+                if let (Some(m), Some(e)) = (r#match, error) {
                     return (
                         Some(Box::new(SingleLineMatch::from_lines(
                             &lines,
-                            r#match.unwrap().offset() + output_offset.unwrap(),
+                            m.offset() + offset,
                             Some("direct regex"),
                         )) as Box<dyn Match>),
                         Some(packet.name),
-                        error,
+                        Some(e),
                         None,
                     );
                 }
@@ -654,13 +752,12 @@ pub fn find_autopkgtest_failure_description(
                 |x| (x.0.iter().map(|x| x.as_str()).collect(), Some(x.1)),
             );
             let (r#match, error) = crate::common::find_build_failure_description(output_lines);
-            let offset = if r#match.is_none() || output_offset.is_none() {
-                summary_offset + packet.offset()
-            } else {
-                r#match.as_ref().unwrap().offset() + output_offset.unwrap()
+            let offset = match (r#match.as_ref(), output_offset) {
+                (Some(m), Some(o)) => m.offset() + o,
+                _ => summary_offset + packet.offset(),
             };
             let description = if let Some(r#match) = r#match.as_ref() {
-                r#match.line()
+                r#match.line().to_string()
             } else if let Some(reason) = packet.reason {
                 format!("Test {} failed: {}", packet.name, reason)
             } else {
@@ -682,9 +779,19 @@ pub fn find_autopkgtest_failure_description(
     (None, None, None, None)
 }
 
-pub fn find_testbed_setup_failure(
-    lines: Vec<&str>,
-) -> (Option<Box<dyn Match>>, Option<Box<dyn Problem>>) {
+/// Searches for testbed setup failures in autopkgtest logs.
+///
+/// This function analyzes log lines to identify errors that occurred during
+/// the autopkgtest testbed setup phase, such as chroot not found errors.
+///
+/// # Arguments
+/// * `lines` - Vector of log lines to analyze
+///
+/// # Returns
+/// A tuple containing:
+/// * An optional match identifying the location of the error
+/// * An optional problem description
+pub fn find_testbed_setup_failure(lines: Vec<&str>) -> AutopkgtestResult {
     for (i, line) in lines.enumerate_backward(None) {
         if let Some((_, command, status_code, stderr)) = lazy_regex::regex_captures!(
             r"\[(.*)\] failed \(exit status ([0-9]+), stderr \'(.*)\'\)\n",
